"""Slovak translations of the program interface"""

MSG_WELCOME_1 = "Víta vás Calcure"
MSG_WELCOME_2 = "Váš kalendár a správca úloh pre terminál!"
MSG_WELCOME_3 = "Súbory s dátami a konfiguráciou boli vytvorené v:"
MSG_WELCOME_4 = "Ak potrebujete viac informácií o podpore, prispievaní atď, navštívte:"
MSG_WELCOME_5 = "Stlačte ? na zobrazenie všetkých klávesových skratiek alebo stlačte akýkoľvek iný kláves na pokračovanie."

TITLE_KEYS_GENERAL = "SPOLOČNÉ SKRATKY"
TITLE_KEYS_CALENDAR = "SKRATKY PRE KALENDÁR"
TITLE_KEYS_JOURNAL  = "SKRATKY PRE DENNÍK"

KEYS_GENERAL = {
        " Medz ": "Prepnúť medzi kalendárom a denníkom",
        "   /   ": "Vypnúť/zapnúť rozdelenie obrazovky",
        "   *   ": "Vypnúť/zapnúť globálne súkromie",
        "   ?   ": "Vypnúť/zapnúť tohto pomocníka",
        "   Q   ": "Reload",
        "   q   ": "Ukončiť",
        }

KEYS_CALENDAR = {
        "  a(A) ": "Pridať (opakujúcu sa) udalosť",
        "   n   ": "Nasledujúci mesiac (deň)",
        "   p   ": "Predchádzajúci mesiac (deň)",
        "   x   ": "Odstrániť udalosť",
        "   r   ": "Premenovať udalosť",
        "  m(M) ": "Presunúť udalosť (v rámci mesiaca)",
        "  g(G) ": "Prejsť na určitý deň (v rámci mesiaca)",
        "   v   ": "Prepnúť medzi zobrazením dní/mesiacov",
        "   h   ": "Nastaviť/zrušiť vysokú prioritu pre udalosť",
        "   l   ": "Nastaviť/zrušiť nízku prioritu pre udalosť",
        "   d   ": "Označiť udalosť ako (ne)dokončenú",
        "   .   ": "Označiť udalosť ako súkromnú/verejnú",
        "   C   ": "Importovať udalosti z calcurse",
        "   R   ": "Vrátiť sa na aktuálny mesiac (deň)",
        }

KEYS_TODO = {
        "  a(A) ": "Pridať novú (pod)úlohu",
        "  h(H) ": "Nastaviť/zrušiť vysokú prioritu pre jednu (všetky) úlohu",
        "  l(L) ": "Nastaviť/zrušiť nízku prioritu pre jednu (všetky) úlohu",
        "  d(D) ": "Označiť jednu (všetky) úlohu ako (ne)dokončenú",
        "  u(U) ": "Zrušiť označenie jednej (všetkých) úlohy",
        "  x(X) ": "Odstrániť jednu (všetky) úlohy (aj s podúlohami)",
        "  t(T) ": "Spustiť/pozastaviť (odstrániť) časovač úlohy",
        "   r   ": "Premenovať úlohu",
        "   s   ": "Prepnúť medzi úlohou a podúlohou",
        "   .   ": "Prepnúť viditeľnosť úlohy",
        "  f(F) ": "Zmeniť (odstrániť) hraničný termín úlohy",
        "   m   ": "Odstrániť úlohu",
        "  C(W) ": "Importovať úlohy z calcurse (taskwarrior)",
        }

MSG_NAME          = "CALCURE"
MSG_VIM           = "Fungujú aj Vim klávesy (j, k, ZZ, ZQ)!"
MSG_INFO          = "Pre viac informácií navštívte:"
MSG_SITE          = "https://anufrievroman.gitbook.io/calcure"
MSG_EXIT          = "Naozaj ukončiť? (y/n) "

MSG_EVENT_HIGH    = "Nastaviť vysokú prioritu udalosti číslo: "
MSG_EVENT_LOW     = "Nastaviť nízku prioritu udalosti číslo: "
MSG_EVENT_DONE    = "Označiť ako hotovú udalosť číslo: "
MSG_EVENT_RESET   = "Obnoviť stav udalosti číslo: "
MSG_EVENT_DEL     = "Odstrániť udalosť číslo: "
MSG_EVENT_REN     = "Premenovať udalosť číslo: "
MSG_NEW_TITLE     = "Zadať nový názov: "
MSG_EVENT_MV      = "Presunúť udalosť číslo: "
MSG_EVENT_MV_TO   = "Presunúť udalosť na (RRRR/MM/DD): "
MSG_EVENT_MV_TO_D = "Presunúť udalosť na: "
MSG_EVENT_DATE    = "Zadajte dátum: "
MSG_EVENT_TITLE   = "Zadajte názov: "
MSG_EVENT_REP     = "Koľkokrát sa udalosť opakuje: "
MSG_EVENT_FR      = "Opakovať udalosť každý (d)eň, týždeň (w), (m)esiac alebo rok (y)? "
MSG_EVENT_IMP     = "Importovať udalosti z Calcurse? (y/n)"
MSG_EVENT_PRIVACY = "Prepnúť viditeľnosť úlohy číslo: "
MSG_TM_ADD        = "Pridať/pozastaviť časovač úlohy číslo: "
MSG_TM_RESET      = "Odstrániť časovač úlohy číslo: "
MSG_TS_HIGH       = "Nastaviť vysokú prioritu úlohe číslo: "
MSG_TS_LOW        = "Nastaviť nízku prioritu úlohe číslo: "
MSG_TS_RES        = "Obnoviť stav úlohy číslo: "
MSG_TS_DONE       = "Označiť ako hotovú úlohu číslo: "
MSG_TS_DEL        = "Odstrániť úlohu číslo: "
MSG_TS_DEL_ALL    = "Naozaj odstrániť všetky úlohy? (y/n)"
MSG_TS_EDT_ALL    = "Do you confirm this action? (y/n)"
MSG_TS_MOVE       = "Presunúť úlohu z čísla: "
MSG_TS_MOVE_TO    = "Presunúť úlohu na číslo: "
MSG_TS_EDIT       = "Upraviť úlohu číslo: "
MSG_TS_TOG        = "Vypnúť/zapnúť podúlohu číslo: "
MSG_TS_SUB        = "Pridať podúlohu úlohe číslo: "
MSG_TS_TITLE      = "Zadajte podúlohu: "
MSG_TS_IM         = "Importovať úlohy z Calcurse? (y/n)"
MSG_TS_TW         = "Importovať úlohy z Taskwarriora? (y/n)"
MSG_TS_NOTHING    = "Nič nie je naplánované..."
MSG_TS_PRIVACY    = "Vypnúť/zapnúť viditeľnosť úlohy číslo: "
MSG_TS_DEAD_ADD   = "Pridať hraničný termín úlohy číslo: "
MSG_TS_DEAD_DEL   = "Odstrániť hraničný termín ulohy číslo: "
MSG_TS_DEAD_DATE  = "Pridať hraničný termín dňa (RRRR/MM/DD): "
MSG_WEATHER       = "Počasie sa načíta..."
MSG_ERRORS        = "Došlo k chybe. Otvorte info.log v priečinku s konfiguráciou."
MSG_INPUT         = "Neplatný vstup."
MSG_GOTO          = "Prejsť na dátum (RRRR/MM/DD): "
MSG_GOTO_D        = "Prejsť na dátum: "

CALENDAR_HINT     = "Medzera · Prepnúť na denník   a · Pridať udalosť  n/p · Zmeniť mesiac ? · Všetky skratky"
CALENDAR_HINT_D   = "Medzera · Prepnúť na denník   a · Pridať udalosť  n/p · Zmeniť deň   ? · Všetky skratky"
JOURNAL_HINT      = "Medzera · Prepnúť na kalendár   a · Pridať úlohu d · Hotovo   i · Dôležité   ? · Všetky skratky"

DAYS = ["PONDELOK", "UTOROK", "STREDA", "ŠTVRTOK", "PIATOK", "SOBOTA", "NEDEĽA"]
DAYS_PERSIAN = ["SHANBEH", "YEKSHANBEH", "DOSHANBEH", "SESHANBEH", "CHAHARSHANBEH", "PANJSHANBEH", "JOMEH"]

MONTHS = ["JANUÁR", "FEBRUÁR", "MAREC", "APRÍL", "MÁJ", "JÚN", "JÚL", "AUGUST", "SEPTEMBER", "OKTÓBER", "NOVEMBER", "DECEMBER"]
MONTHS_PERSIAN = ["FARVARDIN", "ORDIBEHESHT", "KHORDAD", "TIR", "MORDAD", "SHAHRIVAR", "MEHR", "ABAN", "AZAR", "DEY", "BAHMAN", "ESFAND"]
